<?php
declare(strict_types=1);

namespace Inti\Provider;

use Inti\Provider\Query\QueryInterface;
use Inti\Provider\Query\QueryTrait;

use function array_values;
use function count;

/**
 * Default provider implementation.
 *
 * @package Inti\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 */
abstract class AbstractProvider implements
    ProviderInterface,
    QueryInterface
{
    use QueryTrait;

    /** @var array<string, string> */
    protected array $field_map = [
        'id' => 'id',
    ];

    /**
     * Fetches the raw results from the data source.
     *
     * @return array
     */
    abstract protected function fetchResults(): array;

    /**
     * Maps the raw result to the item structure.
     *
     * @param array $result
     * @return array
     */
    protected function mapResultToItem(array $result): array
    {
        return $result;
    }

    /**
     * Maps the raw results array to an array of items.
     *
     * @param array $results
     * @return array
     */
    protected function mapResultsToItems(array $results): array
    {
        foreach ($results as $result) {
            $items[] = $this->mapResultToItem($result);
        }

        return array_values($items ?? []);
    }

    public function getCount(): int
    {
        return count($this->getItems());
    }

    public function getItem(): ?array
    {
        return $this->getItems()[0] ?? null;
    }

    public function getItems(): array
    {
        $this->assertQueryIsInitialized();

        $results = $this->fetchResults();
        return $this->mapResultsToItems($results);
    }
}
